import logging
from typing import Callable
from pyrogram import Client, filters
from pyrogram.types import Message
from dynaconf import settings
from tg_qso_bot.qso_sources.hamlog import HamlogQsoSource
from tg_qso_bot.bot.errors_handling import handle_errors
from tg_qso_bot.bot.app import app, log
from tg_qso_bot.models import Qso

HELP_MESSAGE = """
Синтаксис команды: `/qso <позывной>`.
Бот покажет последние 10 связей с данным позывным.
"""

_logger = logging.getLogger("bot")


def format_qso(q: Qso) -> str:
    return f"{q.call_sign_2:<6} {q.date.strftime('%d.%m.%y'):<8} {q.band:<3} {q.mode}"


def _command_filter(cmd: str, delimiter: str = "@") -> Callable:
    return filters.command([cmd, cmd + delimiter + str(settings.BOT_NAME)])


@app.on_message(_command_filter("qso"))
@handle_errors()
async def request_qso(client: Client, message: Message):
    if len(message.command) < 2:
        reply_message = await message.reply_text(HELP_MESSAGE)
        log.register_reply(message, reply_message)
        return
    callsign = message.command[1].upper()
    user_name = (
        message.from_user.first_name + f" {message.from_user.last_name}"
        if message.from_user.last_name
        else ""
    )
    _logger.info(
        'User "%s" (%d) requested QSO for "%s"', user_name, message.from_user.id, callsign,
    )
    hamlog = HamlogQsoSource()
    qso_list = hamlog.get_qso_list(callsign, limit=10)
    header = f"Последние {len(qso_list)} связей с {callsign}\n"
    table = "\n".join(format_qso(q) for q in qso_list)
    reply_message = await client.send_message(
        message.chat.id,
        f"{header}```{table}```",
        parse_mode="markdown",
        reply_to_message_id=message.message_id,
    )
    log.register_reply(message, reply_message)



from abc import ABC, abstractmethod
from typing import List
from tg_qso_bot.models import Qso


class QsoSource(ABC):
    """
    Storage adapter, from which QSO can be extracted.
    All implemented QSO sources inherit this class.
    """

    @abstractmethod
    def get_qso_list(self, callsign: str, limit: int, skip: int = 0) -> List[Qso]:
        """
        Get QSO list from a source.
        :param callsign: Callsign.
        :param limit: Requested QSO amount.
        :param skip: Indent from top of list.
        :return: List of QSOs.
        """
        raise NotImplementedError()



import asyncio
import sentry_sdk
import pyrogram
from .handlers import *
from .app import app, watchdog


async def main():
    sentry_sdk.init()
    await app.start()
    await asyncio.wait(
        (
            asyncio.create_task(pyrogram.idle()),
            asyncio.create_task(watchdog.run())
        ),
        return_when=asyncio.FIRST_COMPLETED
    )
    await app.stop()


if __name__ == "__main__":
    loop = asyncio.get_event_loop()
    loop.run_until_complete(main())




class ServerResponseError(Exception):
    def __init__(self, url: str, status_code: int, message: str = None):
        self.url = url
        self.status_code = status_code
        self.message = message

    def __repr__(self) -> str:
        return f"Server returns code {self.status_code}."


class ParsingError(Exception):
    pass



import logging


def create_logger(name: str, level=logging.INFO) -> logging.Logger:
    logger = logging.getLogger(name)
    logger.setLevel(level)
    handler = logging.StreamHandler()
    handler.setFormatter(logging.Formatter("%(asctime)s - [%(levelname)s] - %(message)s"))
    logger.addHandler(handler)
    return logger



import requests
from typing import List, Dict, Optional
from datetime import date, datetime
from bs4 import BeautifulSoup
from tg_qso_bot.models import Qso
from .source import QsoSource
from .exceptions import ServerResponseError, ParsingError

HAMLOG_URL = "https://hamlog.online"
HAMLOG_USERAGENT = "Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:84.0) Gecko/20100101 Firefox/84.0"


class _HamlogSession:
    CSRF_TOKEN_FIELD = "csrf-progress"

    def __init__(self, session: requests.Session = None, url: str = HAMLOG_URL, default_headers=None):
        if default_headers is None:
            default_headers = {}
        self._url = url
        self._headers = {"User-Agent": self.CSRF_TOKEN_FIELD, **default_headers}
        self._token: Optional[str] = None
        self._session: requests.Session = session or requests.Session()

    def retrieve_token(self) -> str:
        response = self._session.request("GET", self._url, headers=self._headers)
        if not response.ok:
            raise ServerResponseError(self._url, response.status_code)
        soup = BeautifulSoup(response.text, "html.parser")
        form = soup.find("form", {"action": "/progress.php"})
        if not form:
            raise KeyError("Form with CSRF token not found")
        csrf_field = form.find_next("input", {"name": "csrf-progress"})
        return csrf_field["value"]

    @property
    def token(self) -> str:
        if not self._token:
            self._token = self.retrieve_token()
        return self._token


class HamlogQsoSource(QsoSource):
    def __init__(self, user_agent: str = None):
        self._url = HAMLOG_URL
        self._headers = {"User-Agent": user_agent or HAMLOG_USERAGENT}

    def get_qso_list(self, callsign: str, limit: int, skip: int = 0) -> List[Qso]:
        """
        Get QSO from HamLog.
        :param callsign: Callsign.
        :param limit: Requested QSO amount.
        :param skip: Indent from top of list.
        :return: List of QSOs.
        """
        page = self._get_qso_log_page(callsign)
        qso = self._parse_qso_list(callsign, page)
        return qso[:limit]

    def _get_qso_log_page(self, callsign: str):
        with requests.Session() as session:
            session.verify = False  # disable HamLog SSL certificate verification
            hamlog_session = _HamlogSession(session)
            data = {"callsign": callsign, "csrf-progress": hamlog_session.token}
            response = session.post(f"{self._url}/progress.php", headers=self._headers, data=data)
        if not response.ok:
            raise ServerResponseError(self._url, response.status_code)
        return response.text

    def _parse_qso_list(self, callsign_1: str, data: str) -> List[Qso]:
        # LEGACY CODE!!! Need to rewrite to a modern library
        soup = BeautifulSoup(data, "html.parser")
        result = []
        table = soup.find("table", {"id": "qsos"})
        if not table:
            raise ParsingError("Could not find QSO table", data)
        if thead := table.find("thead"):
            thead.decompose()
        if tfoot := table.find("tfoot"):
            tfoot.decompose()
        for tr in table.find_all("tr"):
            fields = [f.renderContents().strip().decode("utf-8") for f in tr.find_all("td")]
            datetime.strptime(fields[0], "%d %b %Y").date()
            qso = Qso(
                call_sign_1=callsign_1,
                call_sign_2=fields[1],
                date=datetime.strptime(fields[0], "%d %b %Y").date(),
                band=fields[2],
                mode=fields[3],
                report="-",
            )
            result.append(qso)
        return result



import sentry_sdk
from pyrogram import Client
from pyrogram.types import Message

_ERROR_MESSAGE = "😟 Something went wrong.\nAccident ID: {accident_id}"


def _set_event_context(scope: sentry_sdk.Scope, message: Message):
    user = message.from_user
    display_name = user.first_name + f" {user.last_name}" if user.last_name else ""
    scope.set_user(
        {
            "id": message.from_user.id,
            "username": message.from_user.username,
            "display_name": display_name,
        }
    )
    scope.set_context("message", message.__dict__)


def handle_errors(inlude_context: bool = True):
    def without_context(fn):
        async def subwrapper(*args, **kwargs):
            try:
                await fn(*args, **kwargs)
            except Exception as e:
                sentry_sdk.capture_exception(e)
                raise
        return subwrapper

    def with_context(fn):
        async def subwrapper(client: Client, message: Message, *args, **kwargs):
            with sentry_sdk.push_scope() as scope:
                _set_event_context(scope, message)
                try:
                    await fn(client, message, *args, **kwargs)
                except Exception as e:
                    accident_id = sentry_sdk.capture_exception(e, scope=scope)
                    await client.send_message(
                        message.chat.id, _ERROR_MESSAGE.format(accident_id=accident_id)
                    )
                    raise
        return subwrapper

    return with_context if inlude_context else without_context



import asyncio
import pyrogram
from pyrogram.types import Message, Chat
from typing import Awaitable, Callable, List
from .messages_log import MessagesLog

Callback = Callable[["pyrogram.Client", List[Message]], Awaitable]


class MessagesWatchdog:
    """ Watches on logged messages and emit event if they are deleted. """

    def __init__(self, client: "pyrogram.Client", log: MessagesLog):
        self._client = client
        self._log = log
        self._delete_messages_handlers: List[Callback] = []

    def add_delete_messages_handler(self, callback: Callback):
        """ Add handler for delete messages events. """
        self._delete_messages_handlers.append(callback)

    def on_delete_messages(self):
        """ Same as add_delete_messages_handler. """
        def wrapper(fn: Callback) -> Callback:
            self.add_delete_messages_handler(fn)
            return fn
        return wrapper

    async def glance(self):
        deleted_messages: List[Message] = []
        for record in self._log:
            call_message = await self._client.get_messages(record.chat_id, record.message_id)
            if isinstance(call_message, list):
                call_message = call_message[0]
            if call_message.empty:
                call_message.chat = Chat(id=record.chat_id, type="")
                deleted_messages.append(call_message)
        if len(deleted_messages) > 0:
            await self._emit_deleted_messages(deleted_messages)

    async def _emit_deleted_messages(self, messages: List[Message]):
        for handler in self._delete_messages_handlers:
            await handler(self._client, messages)

    async def run(self):
        while True:
            await self.glance()
            await asyncio.sleep(30)
    



from typing import List, Iterator
from dataclasses import dataclass
from pyrogram.types import Message


@dataclass
class LogRecord:
    chat_id: int
    message_id: int
    reply_id: int


class MessagesLog:
    """ Log for outcoming reply messages for futher deletion handling. """

    def __init__(self, buffer_size: int = 200) -> None:
        self._buffer: List[LogRecord] = []
        self._buffer_max_size = buffer_size

    def register_reply(self, to_message: Message, reply: Message):
        """ Register bot reply in log. """
        record = LogRecord(
            chat_id=to_message.chat.id,
            message_id=to_message.message_id,
            reply_id=reply.message_id,
        )
        self._put_record(record)

    def delete_record(self, chat_id: int, message_id: int):
        """ Deletes message from log. """
        for i, record in enumerate(self._buffer):
            if record.chat_id == chat_id and record.message_id == message_id:
                self._buffer.pop(i)
                return
        raise KeyError("Record not found in log")

    def _put_record(self, record: LogRecord) -> int:
        if len(self._buffer) > self._buffer_max_size:
            self._buffer.pop(0)
        self._buffer.append(record)
        return len(self._buffer) - 1

    def __iter__(self) -> Iterator[LogRecord]:
        yield from self._buffer




import logging
import pyrogram
from typing import List
from tg_qso_bot.bot import messages_log
from tg_qso_bot.bot.app import watchdog, log

_logger = logging.getLogger("bot")


async def delete_reply(client: "pyrogram.Client", record: "messages_log.LogRecord"):
    await client.delete_messages(record.chat_id, record.reply_id)
    _logger.info("Deleted reply message #%d in chat #%d", record.reply_id, record.chat_id)


@watchdog.on_delete_messages()
async def on_delete_reply(client: "pyrogram.Client", deleted_messages: List["pyrogram.types.Message"]):
    for deleted_message in deleted_messages:
        if deleted_message.chat is None:
            _logger.warning("Message #%d deleted in unknown chat", deleted_message.message_id)
            continue
        for record in log:
            if deleted_message.message_id == record.message_id and deleted_message.chat.id == record.chat_id:
                await delete_reply(client, record)
                log.delete_record(record.chat_id, record.message_id)





# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['tg_qso_bot',
 'tg_qso_bot.bot',
 'tg_qso_bot.bot.handlers',
 'tg_qso_bot.models',
 'tg_qso_bot.qso_sources',
 'tg_qso_bot.utils']

package_data = \
{'': ['*']}

install_requires = \
['Pyrogram>=1.2.9,<2.0.0',
 'beautifulsoup4>=4.9.1,<5.0.0',
 'dynaconf>=2.2.3,<3.0.0',
 'requests>=2.23.0,<3.0.0',
 'sentry-sdk>=0.19.5,<0.20.0']

entry_points = \
{'console_scripts': ['tg-qso-bot = tg_qso_bot.bot:main']}

setup_kwargs = {
    'name': 'tg-qso-bot',
    'version': '1.3.2',
    'description': 'Simple HAM helper bot written on Python',
    'long_description': '# tg-qso-bot\nSimple HAM helper bot written on Python.\n\n## Features\n\n* Get QSO list from [Hamlog](https://hamlog.ru/) by request from Telegram chat.\n\n## Installation\n```shell script\npython3 -m pip install tg-qso-bot\n```\n\n## Configuration\n1. Get `app_id` and `app_hash` from [Telegram Apps](https://my.telegram.org/apps).\nThe API key is personal and must be kept secret.\n2. Ask [@botfather](https://t.me/botfather) for bot token.\n3. Put `app_id`, `app_hash` and `bot_token` to file `config/settings.json`. You can\ncreate copy of this file for editing called `settings.local.json`.\n\n## Usage\n```shell script\npython3 -m tg_qso_bot.bot\n```\n',
    'author': 'exepirit',
    'author_email': 'you@example.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/exepirit/tg-qso-bot',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from pyrogram import Client
from dynaconf import settings
from .messages_log import MessagesLog
from .messages_watchdog import MessagesWatchdog


app = Client(
    "data",
    api_id=str(settings.MTPROTO_API.APP_ID),
    api_hash=str(settings.MTPROTO_API.APP_HASH),
    bot_token=str(settings.BOT_TOKEN),
)
log = MessagesLog()
watchdog = MessagesWatchdog(app, log)
